/*
 *
 * Copyright  2005, University of Washington, 
 * Department of Computer Science and Engineering. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice, 
 * this list of conditions and the following disclaimer. 
 * 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation 
 * and/or other materials provided with the distribution.
 *
 * 3. Neither name of the University of Washington, Department of Computer 
 * Science and Engineering nor the names of its contributors may be used to 
 * endorse or promote products derived from this software without specific 
 * prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 * ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR 
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL 
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR 
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER 
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 *
 */

#include "SerialCtrl.h"
#include "FTDI466API.h"

namespace ColorSelectorCpp
{

// SerialCtrl::SerialCtrl()
// Default Constructor
SerialCtrl::SerialCtrl()
{
	// FTDI466API is unmanaged C++ so there is no gc (__nogc)
	// create new class object since SerialCtrl is a wrapper to this class
	// method calls will be passed through to usbDevice 
	// or processed in SerialCtrl by calling methods in usbDevice
	device = __nogc new FTDI466API();
	SPIInitialize();
}

// SerialCtrl::~SerialCtrl()
// Default Deconstructor
SerialCtrl::~SerialCtrl()
{
	delete device;
}

// SerialCtrl::SPIInitialize()
// Initializes the device specified by newDevice.
// Returns true on success, false on failure.
bool SerialCtrl::SPIInitialize()
{
	unsigned char txBuffer[256]; // outbound transmission buffer
	unsigned char rxBuffer[256]; // inbount transmission buffer
	DWORD numBytesToSend;
	DWORD numBytesReceived;

	// setup USB device for MPSSE mode
	bool setup = device->open();
	if(!setup)
		return 0;

	cout << "INITIALIZING SPI" << endl;

	// setup for SPI communication
	txBuffer[0] = 0x80; // setup PORT 
	txBuffer[1] = 0x08; // make CS high
	txBuffer[2] = 0x0B; // outputs: SK, DO, CS, inputs:  DI, GPIOL1-L4
	txBuffer[3] = 0x86; // set clk divisor to Tx at 200kHz
	txBuffer[4] = 0x1D; // speed low byte
	txBuffer[5] = 0x00; // speed high byte
	txBuffer[6] = 0x85; // disconnect TDI/DO output from TDO/DI input for loopback testing
	numBytesToSend = 7;

	// send the instructions ot the USB device
	DWORD bytesSent = device->write(txBuffer, numBytesToSend); 

	if(bytesSent != numBytesToSend)
		cerr << "Not all the bytes were sent when initializing MPSSE" << endl;

	// see if there were any error codes when setting up SPI
	DWORD numBytesToRead = device->getReceiveQueueSize();

	if(numBytesToRead > 0)
	{
		numBytesReceived = device->read(rxBuffer, numBytesToRead); 

		if(numBytesReceived != numBytesToRead)
			cerr << "Problem when trying to retrieve the error bytes" << endl;
	
		for(unsigned int i = 0; i < numBytesReceived; i++)
			cout << "Error Byte: " << rxBuffer[i] << endl;

		return false;
	}

	return true;

}

// SerialCtrl::SPIUninitialize()
// Closes the device
void SerialCtrl::SPIUninitialize()
{
	device->close();
}

// SerialCtrl::SPITransaction()
// Creates a data transaction with the specified device.
// Sends the data in value to the device specified by newDevice.  Data received from the device
// is stored in rxBuffer, and the number
// of bytes received is reported in the numBytesReceived variable.
// Returns true on successful transaction, and false on failure
bool SerialCtrl::SPITransaction(unsigned char* txData, unsigned char* rxBuffer, DWORD* numBytesReceived, DWORD* numBytesToBeSent)
{	
	unsigned char txBuffer[255];
	unsigned char hackbuffer[255];

	txBuffer[0] = 0x80; // setup PORT 
	txBuffer[1] = 0x00; // make CS low
	txBuffer[2] = 0x0B; // outputs: SK, DO, CS, inputs:  DI, GPIOL1-L4
	txBuffer[3] = 0x35; // clock out on negative edge, in on negative edge, MSB 
	txBuffer[4] = 0x00; // low byte of length : note a length of zero is 1 byte, 1 is 2 bytes
	txBuffer[5] = 0x00; // high byte of length
	txBuffer[6] = txData[0]; //payload.  You can send more than one byte of payload... how?
	txBuffer[7] = 0x80; // setup PORT 
	txBuffer[8] = 0x08; // make CS high
	txBuffer[9] = 0x0B; // outputs: SK, DO, CS, inputs:  DI, GPIOL1-L4
	DWORD numBytesToSend = 10;
	

	// send the instructions to the USB device
	DWORD bytesSent = device->write(txBuffer, numBytesToSend); 

	if(bytesSent != numBytesToSend)
	{
		cerr << "Not all the bytes were sent" << endl;
		return false;
	}

	Sleep(5); // make sure the usb device has enough time to execute command - 5 ms latency timeout is set

	// see if there were any error codes when setting up SPI
	DWORD numBytesToRead = device->getReceiveQueueSize();

	// store receive bytes from the device, if any
	if(numBytesToRead > 0)
	{
		

		*numBytesReceived = device->read(hackbuffer, numBytesToRead);

		//We only use rxBuffer[0] because only one byte at a time is transmitted
		//the entire buffer [0-255] is available for multibyte transmissions.
		rxBuffer[0] = hackbuffer[0];
		// check to make sure we received it all
		if(*numBytesReceived != numBytesToRead)
		{
			cerr << "Problem when trying to retrieve the bytes from the receive queue" << endl;
			return false;
		}
	}

	return true;
}

// SerialCtrl::DecodeInput
// Decode the input received from the USB device
void SerialCtrl::DecodeInput(unsigned char* rxBuffer, int* xSpeed, int* ySpeed, int* leftButtonPressed, int* rightButtonPressed)
{
	//You must implement your protocol for mouse value decoding here.

}

}
